# #
# frankp, 2.04.1995 #
# #

#++
	Dom::Complex  --  the field of complex numbers

	The test whether an expression is complex is performed by 
	calling 'rectform'.

	The domain has category Cat::Field, which is very optimistic 
	(indeed false, because for example bool(1.0 = float(3) / float(3))
	FALSE. The category is assumed only because of pragmatism.
	Note that elements may not have an unic representation, for example
	bool( 0 = sin(2)^2 + cos(2)^2 - 1 ) returns FALSE.

	Note: Unknowns defined in "globalReal" of 'rectform' do not 
        influence the process of the testing of Complex, i.e. the set is 
        redefined to the empty set during the test.
++#

Dom::Complex := DomainConstructor::new(

# Name = # Dom::Complex,

# Args = # [ ],

# Local Paramameters = # [ ],

# Init = # (
    if args(0) <> 0 then
	error("no arguments expected")
    end_if),

# Super Domains = # Dom::BaseDomain,

# Categories = # [ Cat::DifferentialRing, Cat::Field ],

# Axioms = # [ ],

# Methods = #

"new" = proc(x)
begin
    x:= this::convert(x);
    if x = FAIL then error("invalid argument") else x end_if
end_proc,

"D" = 0,

"diff" = fun((if args(0) = 1 then args(1) else 0 end_if)),

"characteristic" = 0,

"convert" = proc(x)
    local d, t;
begin
    d:= rectform::globalReal;
    sysassign(rectform::globalReal, {});

    t:= rectform( expr(x),{} );
    if op(t,3) = 0 and not has(t,{hold(Re),hold(Im)}) then
        sysassign(rectform::globalReal, d);
        new(this,x)
    else 
        sysassign(rectform::globalReal, d);
	FAIL
    end_if
end_proc,

"convert_to" = proc(x,T)
begin
    if domtype(T) <> DOM_DOMAIN then T:= domtype(T) end_if;

    case T
    of this 	     do
    of Dom::Numeric  do
    of DOM_COMPLEX   do
    of DOM_EXPR	     do return( x )
    otherwise
	if T::constructor = Dom::ExpressionField then
	    return( T::new( extop(x,1) ) )
	else
	    return( FAIL )
	end_if
    end_case
end_proc,

"random" = fun(new(this,random(args()))),

"_plus" = proc()
    local t, o;
begin
    t:= [args()];
    o:= select( t,fun(bool(domtype(args(1))=this)) );
    t:= select( t,fun(bool(domtype(args(1))<>this)) );
    t:= _plus( op(t), op(map(o,extop,1)) );
    this::convert( t )
end_proc,

"_mult" = proc()
    local t, o;
begin
    t:= [args()];
    o:= select( t,fun(bool(domtype(args(1))=this)) );
    t:= select( t,fun(bool(domtype(args(1))<>this)) );
    t:= _mult( op(t), op(map(o,extop,1)) );
    this::convert( t )
end_proc,

"_power" = proc(x,y)
begin
    if domtype(x) = this and domtype(y) = this then
	this::convert( extop(x,1)^extop(y,1) )
    elif domtype(x) = this then
	this::convert( extop(x,1)^y )
    else
	this::convert( x^extop(y,1) )
    end_if
end_proc,

"invert" = fun(new(this,1/extop(args(1),1))),

"negate" = fun(new(this,-extop(args(1),1))),

"zero" = 0,

"one" = 1,

"expr" = fun(extop(args(1),1)),

"Re" = fun(new(this,Re(extop(args(1),1)))),

"Im" = fun(new(this,Im(extop(args(1),1)))),

"iszero" = fun(iszero(extop(args(1),1))),

"equal" = fun(_equal(extop(args(1),1),extop(args(2),1)))


)():				

# end of file #
